const nullOrUndefinedConversions = new Map([
    ['null', null],
    ['undefined', undefined]
]);
export function unserializeObjectKey(serialized) {
    const result = {};
    if (typeof serialized !== 'object')
        throw new Error('The Argument is invalid, please make sure you are passing an object as the argument.');
    Object.entries(serialized).forEach(([key, value]) => {
        const keys = key.split(/[\.\[\]]+/).filter(Boolean);
        let current = result;
        let indicatorWithUndefinedKey = '';
        const lastKeyElement = keys[keys.length - 1];
        if (lastKeyElement === '()' || lastKeyElement === '!!')
            indicatorWithUndefinedKey = keys.pop();
        const lastKeysIndexAfterPopping = keys.length - 1;
        keys.forEach((k, index) => {
            const isArray = index < lastKeysIndexAfterPopping && /^[\d]+$/.test(keys[index + 1]);
            if (index === lastKeysIndexAfterPopping) {
                current[denormalizePrimitiveKey(k)] = valueParserByKey(k + indicatorWithUndefinedKey, value);
            }
            else {
                if (!current[k]) {
                    current[k] = isArray ? [] : {};
                }
                current = current[k];
            }
        });
    });
    return result;
}
export function serializeObjectKey(unserialized) {
    const acc = {};
    if (typeof unserialized !== 'object')
        throw new Error('The Argument is invalid, please make sure you are passing an object as the argument.');
    const recursive = (data, parentKey = '') => {
        if (Array.isArray(data)) {
            data.forEach((item, index) => {
                const currentKey = parentKey ? `${parentKey}[${index}]` : '';
                recursive(item, currentKey);
            });
        }
        else if (typeof data === 'object' && data !== null) {
            Object.entries(data).forEach(([key, value]) => {
                const currentKey = parentKey ? `${parentKey}.${key}` : key;
                recursive(value, currentKey);
            });
        }
        else
            acc[normalizePrimitiveKey(parentKey, data)] = data;
    };
    recursive(unserialized);
    return acc;
}
export function normalizePrimitiveKey(key, value) {
    switch (typeof value) {
        case 'number':
            return `${key}()`;
        case 'boolean':
            return `${key}!!`;
        default:
            return key;
    }
}
export function denormalizePrimitiveKey(key) {
    const suffixes = ['()', '!!'];
    for (const suffix of suffixes) {
        if (key.endsWith(suffix)) {
            return key.slice(0, -suffix.length);
        }
    }
    return key;
}
export function valueParserByKey(key, value) {
    if (key.endsWith('()') || key.endsWith('!!'))
        return JSON.parse(value);
    return nullOrUndefinedConversions.has(value) ? nullOrUndefinedConversions.get(value) : value;
}
